/*
 * mdtupla.c
 *      Reserva i alliberament de memoria per tipus tupla
 */

/*
 * Inclusions estandard
 */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Definicio de constants
 */
#define M 10

/*
 * Definicio de tipus
 */
typedef int TaulaEnter[M];

typedef struct
{
    char c;
    int i;
    TaulaEnter t;
}
TuplaA;

typedef struct
{
    float x, y;
}
TuplaB;

/*
 * Prototipus
 */
TuplaA *CrearTuplaA(void);
void DestruirTuplaA(TuplaA * t);
void EscriureTuplaA(TuplaA t);

TuplaB *CrearTuplaB(void);
void DestruirTuplaB(TuplaB * t);
void EscriureTuplaB(TuplaB t);

void EscriureCaracter(char c);
void EscriureEnter(int i);
void EscriureReal(float f);

/*
 * Programa principal
 */
int
main(void)
{
    TuplaA *ta;
    TuplaB *tb;
    int i;

    /*
     * TuplaA
     */
    ta = CrearTuplaA();
    (*ta).c = 'v';
    (*ta).i = 321;
    i = 0;
    while (i < M) {
	(*ta).t[i] = 2 * i;
	i = i + 1;
    }
    EscriureTuplaA(*ta);
    DestruirTuplaA(ta);
    /*
     * TuplaB
     */
    tb = CrearTuplaB();
    (*tb).x = 3.7;
    (*tb).y = -0.1e-4;
    EscriureTuplaB(*tb);
    DestruirTuplaB(tb);

    return EXIT_SUCCESS;
}

/*
 * Implementacio dels subprogrames
 */
TuplaA *
CrearTuplaA(void)
{
    TuplaA *t;

    t = malloc(sizeof (TuplaA));
    assert(t != NULL);
    return t;
}

void
DestruirTuplaA(TuplaA * t)
{
    free(t);
}

void
EscriureTuplaA(TuplaA t)
{
    int i;

    EscriureCaracter(t.c);
    EscriureCaracter(' ');
    EscriureEnter(t.i);
    EscriureCaracter(' ');
    i = 0;
    while (i < M) {
	EscriureEnter(t.t[i]);
	EscriureCaracter(' ');
	i = i + 1;
    }
    EscriureCaracter('\n');
}

TuplaB *
CrearTuplaB(void)
{
    TuplaB *t;

    t = malloc(sizeof (TuplaB));
    assert(t != NULL);
    return t;
}

void
DestruirTuplaB(TuplaB * t)
{
    free(t);
}

void
EscriureTuplaB(TuplaB t)
{
    EscriureCaracter('(');
    EscriureReal(t.x);
    EscriureCaracter(',');
    EscriureReal(t.y);
    EscriureCaracter(')');
    EscriureCaracter('\n');
}

void
EscriureCaracter(char c)
{
    putchar(c);
}

void
EscriureEnter(int i)
{
    int ret;

    ret = printf("%d", i);
    assert(ret > 0);
}

void
EscriureReal(float f)
{
    int ret;

    ret = printf("%g", f);
    assert(ret > 0);
}
